﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.EnterpriseServices;
using System.Runtime.InteropServices;

namespace Newsletter
{
    [Transaction(TransactionOption.RequiresNew)]
    [ClassInterface(ClassInterfaceType.AutoDual)]
    public class PersistenceManager : ServicedComponent, IPersistenceManager
    {
        private SubscriberDataClassesDataContext dc = new SubscriberDataClassesDataContext();

        public PersistenceManager()
        {
        }

        #region IPersistenceManager Member
        
        [AutoComplete]
        public SubscriberBO Insert(string alias, string email)
        {
            Subscriber s = new Subscriber();
            s.Id = Guid.NewGuid();
            s.Alias = alias;
            s.Email = email;
            s.Committed = false;
            dc.Subscriber.InsertOnSubmit(s);
            dc.SubmitChanges();

            // Wird hier eine Exception geworfen ist trotz SubmitChanges nichts
            // in der DB. SubmitChanges generiert nur das erforderliche SQL,
            // die Transaktion läuft aber noch.
            // throw new Exception("Transaction fault");

            return new SubscriberBO(s.Alias, s.Email, s.Committed, s.Id.ToString());
        }

        [AutoComplete]
        public SubscriberBO FindById(string id)
        {
            Subscriber s = (from sub in dc.Subscriber where sub.Id.Equals(new Guid(id)) select sub).First();
            return new SubscriberBO(s);
        }

        [AutoComplete]
        public SubscriberBO[] FindAllCommitted()
        {
            IEnumerable<SubscriberBO> subscribers = (from s in dc.Subscriber where s.Committed.Equals(true) select new SubscriberBO(s));
            return subscribers.ToArray<SubscriberBO>();
        }

        [AutoComplete]
        public SubscriberBO[] FindAll()
        {
            IEnumerable<SubscriberBO> subscribers = from s in dc.Subscriber select new SubscriberBO(s);
            return subscribers.ToArray<SubscriberBO>();
        }

        [AutoComplete]
        public void CommitSubscriber(string id)
        {
            Subscriber s = (from sub in dc.Subscriber where sub.Id.Equals(new Guid(id)) select sub).First();
            s.Committed = true;
            dc.SubmitChanges();
        }

        #endregion
    }
}
